@@ -1,6 +1,8 @@
 from .cli import cli
 from .version import __version__
 from .exceptions import *
+from .stages import Stages
+from .contexts import Contexts
 from .config import Configs
 from .providers import ProviderBase, StoreProvider, ProviderManager, Providers
 from .parameters import ParameterProvider, ParameterManager, Parameters
