@@ -35,18 +35,15 @@ class ParameterManager():
         project = Configs.project
         application = Configs.application
         provider = Providers.ParameterProvider()
-        if uninherited:
-            Logger.info(f"Start listing uninherited SSM parameters: project={project}, application={application}, stage={Stages.shorten(stage)}")
-        else:
-            Logger.info(f"Start listing SSM parameters: project={project}, application={application}, stage={Stages.shorten(stage)}")
+        Logger.info(f"Start listing SSM parameters: project={project}, application={application}, stage={Stages.shorten(stage)}")
         return provider.list(project, application, stage, uninherited)
 
     def add(self, stage, key, value):
         self.validate_key(key)
         project = Configs.project
         application = Configs.application
-        provider = Providers.ParameterProvider()
         Logger.info(f"Start adding parameter: project={project}, application={application}, stage={Stages.shorten(stage)}, key={key}, value={value}")
+        provider = Providers.ParameterProvider()
         return provider.add(project, application, stage, key, value)
 
     def get(self, stage, key, revision):
