@@ -1,11 +1,5 @@
-import os
 import re
-import yaml
-import json
-import pathlib
-import numbers
 import boto3
-from botocore.exceptions import ClientError
 from dsocli.exceptions import DSOException
 from dsocli.logger import Logger
 from dsocli.config import Configs
@@ -14,14 +8,21 @@ from dsocli.secrets import SecretProvider
 from dsocli.stages import Stages
 from dsocli.constants import *
 from dsocli.dict_utils import set_dict_value
+from dsocli.contexts import Contexts
+from dsocli.aws_utils import *
 
 
 _default_spec = {
+    'prefix': '/dso/v1'
 }
 
 def get_default_spec():
     return _default_spec.copy()
 
+_settings = {
+    'id': 'secret/aws/ssm/v1',
+}
+
 
 session = boto3.session.Session()
 ssm = session.client(
@@ -32,24 +33,12 @@ ssm = session.client(
 class AwsSsmSecretProvider(SecretProvider):
 
     def __init__(self):
-        super().__init__('secret/aws/ssm/v1')
+        super().__init__(_settings['id'])
 
-###--------------------------------------------------------------------------------------------
 ###--------------------------------------------------------------------------------------------
 
-    def get_secret_prefix(self, project, application, stage, key=None):
-        result = "/dso/secrets"
-        result += f"/{project}"
-        ### every application must belong to a project, no application overrides allowed in the default project
-        if not project == 'default':
-            result += f"/{application}"
-        else:
-            result += "/default"
-        stage = Stages.normalize(stage)
-        result += f"/{stage}"
-        if key:
-            result += f"/{key}"
-        return result
+    def get_prefix(self):
+        return Configs.parameter_spec('prefix') + '/parameters'
 
 ###--------------------------------------------------------------------------------------------
 
@@ -63,8 +52,7 @@ class AwsSsmSecretProvider(SecretProvider):
         Logger.debug(f"Checking secret overwrites: project={project}, application={application}, stage={stage}, key={key}")
         
         ### check children secrets
-        path = self.get_secret_prefix(project, application, stage, key)
-        # secrets = ssm.describe_parameters(ParameterFilters=[{'Key':'Type','Values':['SecureString']},{'Key':'Name', 'Option': 'BeginsWith', 'Values':[f"{path}."]}])
+        path = self.get_prefix(project, application, stage, key)
         response = ssm.describe_parameters(ParameterFilters=[{'Key':'Name', 'Option': 'BeginsWith', 'Values':[f"{path}."]}])
         if len(response['Parameters']) > 0:
             raise DSOException("Secret key '{0}' is not allowed in the given context becasue it would overwrite '{0}.{1}' and all other secrets in '{0}.*' scope if any.".format(key,response['Parameters'][0]['Name'][len(path)+1:]))
@@ -73,125 +61,57 @@ class AwsSsmSecretProvider(SecretProvider):
         scopes = key.split('.')
         for n in range(len(scopes)-1):
             subKey = '.'.join(scopes[0:n+1])
-            path = self.get_secret_prefix(project, application, stage, subKey)
+            path = self.get_prefix(project, application, stage, subKey)
             Logger.debug(f"Describing secrets: path={path}")
             # secrets = ssm.describe_parameters(ParameterFilters=[{'Key':'Type', 'Values':['SecureString']},{'Key':'Name', 'Values':[path]}])
             response = ssm.describe_parameters(ParameterFilters=[{'Key':'Name', 'Values':[path]}])
             if len(response['Parameters']) > 0:
                 raise DSOException("Secret key '{0}' is not allowed in the given context becasue it would overwrite secret '{1}'.".format(key, subKey))
 
-###--------------------------------------------------------------------------------------------
-
-    def locate_secret(self, project, application, stage, key, uninherited=False):
-        Logger.debug(f"Locating SSM secret: project={project}, application={application}, stage={stage}, key={key}")
-        paths = self.get_hierachy_paths(project, application, stage, key, uninherited)
-        # Logger.debug(f"SSM paths to search in order: {paths}")
-        for path in paths:
-            Logger.debug(f"Describing SSM secrets: path={path}")
-            # result = ssm.describe_parameters(ParameterFilters=[{'Key':'Type','Values':['SecureString']},{'Key':'Name', 'Values':[path]}])
-            response = ssm.describe_parameters(ParameterFilters=[{'Key':'Name', 'Values':[path]}])
-            if len(response['Parameters']) > 0: return response['Parameters']
-        return []
-
-###--------------------------------------------------------------------------------------------
-
-    def load_ssm_path(self, secrets, path, decrypt, recurisve=True):
-        Logger.debug(f"Loading SSM secrets: path={path}")
-        p = ssm.get_paginator('get_parameters_by_path')
-        paginator = p.paginate(Path=path, Recursive=recurisve, WithDecryption=decrypt, ParameterFilters=[{'Key': 'Type','Values': ['SecureString']}]).build_full_result()
-        for secret in paginator['Parameters']:
-            key = secret['Name'][len(path)+1:]
-            value = secret['Value']
-            if key in secrets:
-                Logger.warn("Inherited secret '{0}' overridden.".format(key))
-            secrets[key] = {'Value': secret['Value'], 
-                                'Path': secret['Name'],
-                                'Version': secret['Version'],
-                                'Date': secret['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
-                            }
-        return secrets
-
-###--------------------------------------------------------------------------------------------
-
-    def get_hierachy_paths(self, project, application, stage, key, uninherited):
-        paths = []
-        if uninherited:
-            paths.append(self.get_secret_prefix(project, application, stage, key))
-        else:
-            ### Add the global context: /dso/default/default/default/0
-            paths.append(self.get_secret_prefix('default', 'default', 'default/0', key))
-            if not Stages.is_default(stage):
-                ### Add global stage context
-                paths.append(self.get_secret_prefix('default', 'default', Stages.get_stage_default_env(stage), key))
-                ### Add global numbered stage context
-                if not Stages.is_stage_default_env(stage):
-                    paths.append(self.get_secret_prefix('default', 'default', stage, key))
-
-            if not project == 'default':
-                ### Add the project context: /dso/project/default/default/0
-                paths.append(self.get_secret_prefix(project, 'default', 'default/0', key))
-                if not Stages.is_default(stage):
-                    ### Add the project stage context: /dso/project/default/stage/0
-                    paths.append(self.get_secret_prefix(project, 'default', Stages.get_stage_default_env(stage), key))
-                    ### Add the project numbered stage context: /dso/project/default/stage/env
-                    if not Stages.is_stage_default_env(stage):
-                        paths.append(self.get_secret_prefix(project, 'default', stage, key))
-                
-                if not application == 'default':
-                    ### Add the application context: /dso/project/application/default/0
-                    paths.append(self.get_secret_prefix(project, application, 'default/0', key))
-                    if not Stages.is_default(stage):
-                        ### Add the project stage context: /dso/project/application/stage/0
-                        paths.append(self.get_secret_prefix(project, application, Stages.get_stage_default_env(stage), key))
-                        ### Add the application numbered stage context: /dso/project/application/stage/env
-                        if not Stages.is_stage_default_env(stage):
-                            paths.append(self.get_secret_prefix(project, application, stage, key))
-
-        return paths
-
 ###--------------------------------------------------------------------------------------------
 
     def list(self, project, application, stage, uninherited, decrypt):
-        ### construct search path in hierachy with no key specified
-        paths = list(self.get_hierachy_paths(project, application, stage, None, uninherited))
-        # Logger.debug(f"SSM paths to search in order: {paths}")
-        secrets = {}
-        for path in paths:
-            self.load_ssm_path(secrets, path, decrypt)
-
+        secrets = load_context_ssm_parameters(project, application, stage, ['SecureString'], prefix=self.get_prefix(), uninherited=uninherited, decrypt=decrypt)
         result = {'Secrets': []}
-        for item in secrets.items():
-            result['Secrets'].append({'Key': item[0], 
-                                        'Value': item[1]['Value'], 
-                                        'Path': item[1]['Path'],
-                                        'RevisionId': str(item[1]['Version']),
-                                        'Date': item[1]['Date'],
-                                        # 'Version': item[1]['Version'],
-                                        })
+        for key, details in secrets.items():
+            result['Secrets'].append({
+                                'Key': key, 
+                                'Value': details['Value'], 
+                                'Scope': details['Scope'],
+                                'RevisionId': str(details['Version']),
+                                'Date': details['Date'],
+                                'Version': details['Version'],
+                                'Path': details['Path'],
+                                })
 
         return result
 
 ###--------------------------------------------------------------------------------------------
 
     def add(self, project, application, stage, key, value):
-        self.assert_no_scope_overwrites(project, application, stage, key)
-        found = self.locate_secret(project, application, stage, key, uninherited=True)
+        Logger.debug(f"Checking SSM secret overwrites: project={project}, application={application}, stage={stage}, key={key}")
+        assert_ssm_parameter_no_namespace_overwrites(project, application, stage, key, prefix=self.get_prefix())
+        Logger.debug(f"Locating SSM secret: project={project}, application={application}, stage={stage}, key={key}")
+        found = locate_ssm_parameter_in_context_hierachy(project, application, stage, key, prefix=self.get_prefix(), uninherited=True)
         if found and not found[0]['Type'] == 'SecureString':
             raise DSOException(f"Failed to add secret '{key}' becasue becasue the key is not available in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
-        path = self.get_secret_prefix(project, application, stage=stage, key=key)
+        path = get_ssm_path(project, application, stage=stage, key=key, prefix=self.get_prefix())
         Logger.info(f"Adding SSM secret: path={path}")
         response = ssm.put_parameter(Name=path, Value=value, Type='SecureString', Overwrite=True)
-        return {'Key': key, 
+        return {
+                'Key': key, 
                 'Value': value,
-                'Path': path,
+                'Scope': Contexts.translate_context(project, application, stage),
                 'RevisionId': str(response['Version']),
-                # 'Version': response['Version'],
+                'Version': response['Version'],
+                'Path': path,
                 }
 
 ###--------------------------------------------------------------------------------------------
 
     def get(self, project, application, stage, key, revision=None):
-        found = self.locate_secret(project, application, stage, key)
+        Logger.debug(f"Locating SSM secret: project={project}, application={application}, stage={stage}, key={key}")
+        found = locate_ssm_parameter_in_context_hierachy(project, application, stage, key, prefix=self.get_prefix())
         if not found:
             raise DSOException(f"Secret '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
         else:
@@ -206,24 +126,26 @@ class AwsSsmSecretProvider(SecretProvider):
                     'RevisionId': str(secrets[0]['Version']),
                     'Key': key, 
                     'Value': secrets[0]['Value'],
-                    'Path': found[0]['Name'],
+                    'Scope': Contexts.translate_context(project, application, stage),
                     'Date': secrets[0]['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
                     'User': secrets[0]['LastModifiedUser'],
-                    # 'Version': secrets[0]['Version'],
+                    'Version': secrets[0]['Version'],
+                    'Path': found[0]['Name'],
                     }
         else:
             ### get specific revision
             secrets = [x for x in secrets if str(x['Version']) == revision]
             if not secrets:
-                raise DSOException(f"Revision '{revision}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}, key={key}")
+                raise DSOException(f"Revision '{revision}' not found for secret '{key}' in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
             result = {
                     'RevisionId': str(secrets[0]['Version']),
                     'Key': key, 
                     'Value': secrets[0]['Value'],
-                    'Path': found[0]['Name'],
+                    'Scope': Contexts.translate_context(project, application, stage),
                     'Date': secrets[0]['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
                     'User': secrets[0]['LastModifiedUser'],
-                    # 'Version': secrets[0]['Version'],
+                    'Version': secrets[0]['Version'],
+                    'Path': found[0]['Name'],
                     }
 
         return result
@@ -231,7 +153,7 @@ class AwsSsmSecretProvider(SecretProvider):
 ###--------------------------------------------------------------------------------------------
 
     def history(self, project, application, stage, key):
-        found = self.locate_secret(project, application, stage, key)
+        found = locate_ssm_parameter_in_context_hierachy(project, application, stage, key, prefix=self.get_prefix())
         if not found:
             raise DSOException(f"Secret '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
         else:
@@ -245,10 +167,11 @@ class AwsSsmSecretProvider(SecretProvider):
                 'RevisionId': str(secret['Version']),
                 'Key': key,
                 'Value': secret['Value'],
-                'Path': found[0]['Name'],
+                'Scope': Contexts.translate_context(project, application, stage),
                 'Date': secret['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
                 'User': secret['LastModifiedUser'],
-                # 'Version': secret['Version'],
+                'Version': secret['Version'],
+                'Path': found[0]['Name'],
             } for secret in secrets]
         }
 
@@ -257,8 +180,9 @@ class AwsSsmSecretProvider(SecretProvider):
 ###--------------------------------------------------------------------------------------------
 
     def delete(self, project, application, stage, key):
-        ### only secrets owned by the stage can be deleted, hence uninherited=True
-        found = self.locate_secret(project, application, stage, key, uninherited=True)
+        Logger.debug(f"Locating SSM secret: project={project}, application={application}, stage={stage}, key={key}")
+        ### only secrets owned by the context can be deleted, hence uninherited=True
+        found = locate_ssm_parameter_in_context_hierachy(project, application, stage, key, prefix=self.get_prefix(), uninherited=True)
         if not found:
                 raise DSOException(f"Secret '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
         else:
@@ -270,6 +194,7 @@ class AwsSsmSecretProvider(SecretProvider):
         ssm.delete_parameter(Name=found[0]['Name'])
         return {
                 'Key': key, 
+                'Scope': Contexts.translate_context(project, application, stage),
                 'Path': found[0]['Name'],
                 }
 
