@@ -33,10 +33,7 @@ class SecretManager():
         project = Configs.project
         application = Configs.application
         provider = Providers.SecretProvider()
-        if uninherited:
-            Logger.info(f"Start listing uninherited SSM secrets: project={project}, application={application}, stage={Stages.shorten(stage)}")
-        else:
-            Logger.info(f"Start listing SSM secrets: project={project}, application={application}, stage={Stages.shorten(stage)}")
+        Logger.info(f"Start listing SSM secrets: project={project}, application={application}, stage={Stages.shorten(stage)}")
         return provider.list(project, application, stage, uninherited, decrypt)
 
     def add(self, stage, key, value):
